; ##############################################################################
; ##############################################################################
; ##############################################################################
;																			
; Timer1 (16Bit ) Vorteiler 												
;																			
; TCCR1B ==> ICNC1 ICES1 RESERVIERT WGM13 WGM12 CS12 CS11 CS10 				
;																			
;CS12 CS11 CS10 Description													
;	0	0 	0 	No clock source. (Timer/Counter stopped)					
;	0 	0 	1 	clkI/O/1    (No prescaling)									
;	0 	1 	0 	clkI/O/8    (From prescaler)								
; *	0 	1 	1 	clkI/O/64   (From prescaler)								
; 	1 	0 	0 	clkI/O/256  (From prescaler)								
; 	1 	0 	1 	clkI/O/1024 (From prescaler)								
;	1 	1 	0 	External clock source on T1 pin. Clock on falling edge.		
;	1 	1 	1 	External clock source on T1 pin. Clock on rising edge.		
;																			
TIMER1_INITIALISIERUNG:
	ldi temp,(0<<CS12)|(1<<CS11) |(1<<CS10)
	out TCCR1B,temp          
	; Timer 1 Startwert 	
	ldi temp,HIGH(TIMERWERT1)
	out tcnt1H,temp         
	ldi temp,LOW (TIMERWERT1) 
	out tcnt1L,temp     
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Timer 0+1+2 berlauf-Int aktivieren (Timer/Counter Interrupt Mask)		
;																			
; TIMSK ==> OCIE2 TOIE2 TICIE1 OCIE1A OCIE1B TOIE1 R TOIE0  				
;																			
TIMER_AKTIVIEREN:
	ldi temp,(0<<OCIE2)|(0<<TOIE2)|(0<<TICIE1)|(0<<OCIE1A)|(0<<OCIE1B)|(1<<TOIE1)|(0<<TOIE0)
	out TIMSK,temp   
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; bei 16 MHz ==> 10ms 															
TIM1_OVF:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push TEMP1
	push TEMP2
	push ZL
	push ZH
; ----------------------------------
; Timer Startwert	 				
	ldi temp,HIGH(TIMERWERT1)
	out tcnt1H,temp         
	ldi temp,LOW (TIMERWERT1) 
	out tcnt1L,temp
; ----------------------------------
	rcall VORTEILER
	rcall DIGIT
; ----------------------------------
	pop ZH
	pop ZL
	pop TEMP2
	pop TEMP1
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 				
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Vorteiler 10:1																
VORTEILER:
	LDS temp,(adr_ssVT)		; aus SRAM laden			
	inc temp				; +1						
	STS (adr_ssVT),temp		; in SRAM speichern			
	cpi temp,10				; vergleich mit 10			
	brsh VORTEILER_w		; ist gleich oder grer	
	STS (adr_ssVT),temp		; in SRAM speichern			
	ret	
VORTEILER_w:
	STS (adr_ssVT),NULL		; in SRAM speichern			
	; ZEIT				
	rcall ZEIT
	; DCF-UHR			
	rcall DCF_UHR
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ZEIT:
; 100-el Sekunden		
	LDS temp,(adr_ss100)	; aus SRAM laden			
	inc temp				; +1						
	cpi temp,100			; vergleich mit 100			
	brsh ZEIT_SS			; ist gleich oder grer	
	STS (adr_ss100),temp	; in SRAM speichern			
	ret						; zurck					
; Sekunden				
ZEIT_SS:
	STS(adr_ss100),NULL
	; PIN-WERT (fr Automatikmodus)
	LDS temp,(adr_DCF_PIN_WERT)
	STS(adr_DCF_PIN_WERT_OLD),temp
	STS(adr_DCF_PIN_WERT),NULL
	; PROGRAMMFREIGABE	
	STS(adr_FREIGABE_PROGRAMM),EINS
	; DCF PHASE TIMER	
	rcall TIMER_PHASE
	LDS temp,(adr_ss)	; aus SRAM laden	
	inc temp
	cpi temp,60
	brsh ZEIT_MM
	STS (adr_ss),temp	; in SRAM speichern	
	ret	
; Minuten				
ZEIT_MM:
	STS (adr_ss),NULL	; in SRAM speichern	

	LDS temp,(adr_mm)	; aus SRAM laden	
	inc temp
	cpi temp,60
	brsh ZEIT_HH
	STS (adr_mm),temp	; in SRAM speichern	
	ret	
; Stunden				
ZEIT_HH:
	STS (adr_mm),NULL	; in SRAM speichern	

	LDS temp,(adr_hh)	; aus SRAM laden	
	inc temp
	cpi temp,24
	brsh ZEIT_TAG
	STS (adr_hh),temp	; in SRAM speichern	
	ret	
; TAG					
ZEIT_TAG:
	; ROTDCF			
	cbi (LED_DCF_ROT_PORT),(LED_DCF_ROT_PIN_NR)	 
	STS (adr_hh),NULL	; in SRAM speichern	

; WOCHENTAG				
	LDS temp,(adr_WOCHENTAG)
	inc temp
	cpi temp,7+1
	brlo ZEIT_WOCHENTAG_w
	ldi temp,1
ZEIT_WOCHENTAG_w:
	STS (adr_WOCHENTAG),temp

	LDS temp,(adr_TAG)	; aus SRAM laden	
	inc temp
	STS (adr_TAG),temp	; in SRAM speichern	
	
	rcall ZEIT_TAGE_IM_MONAT
	cp temp,temp1 
	brsh ZEIT_MONAT
	ret
; MONAT					
ZEIT_MONAT:
	STS (adr_TAG),EINS

	LDS temp,(adr_MONAT); aus SRAM laden	
	inc temp
	cpi temp,13 
	brsh ZEIT_JAHR
	STS (adr_MONAT),temp; in SRAM speichern	
	ret
; JAHR					
ZEIT_JAHR:
	STS (adr_MONAT),EINS

	LDS temp,(adr_JAHR)	; aus SRAM laden	
	inc temp
	STS (adr_JAHR),temp	; in SRAM speichern	

ret
; ##############################################################################
; Anzahl der TAGE im Monat	
ZEIT_TAGE_IM_MONAT:
	LDS temp1,(adr_MONAT)
	; vergleich	
	cpi temp1,2			; Februar 28 	
	breq ZEIT_TAGE_IM_MONAT_28
	cpi temp1,4			; April 30		
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,6			; Juni 30		
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,8			; August 31		
	breq ZEIT_TAGE_IM_MONAT_31
	cpi temp1,9			; September 30	
	breq ZEIT_TAGE_IM_MONAT_30
	cpi temp1,11		; November 30	
	breq ZEIT_TAGE_IM_MONAT_30
ZEIT_TAGE_IM_MONAT_31:
	ldi temp1,31
ret
ZEIT_TAGE_IM_MONAT_28:
	; Jahr laden					
	LDS temp1,(adr_JAHR)
	; SCHKEIFE						
ZEIT_TAGE_IM_MONAT_28s:
	; -4							
	subi temp1,4
	; vergleich						
	tst temp1
	; NULL=Schaltjahr				
	breq ZEIT_TAGE_IM_MONAT_29
	brpl ZEIT_TAGE_IM_MONAT_28s
	; negativ kein Schaltjahr		
	ldi temp1,28
	ret
ZEIT_TAGE_IM_MONAT_29:
	ldi temp1,29
	ret
ZEIT_TAGE_IM_MONAT_30:
	ldi temp1,30
	ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TIMER_PHASE:
	LDS temp,(adr_DCF_PHASE_TIMER)
	tst temp
	breq TIMER_PHASE_w
	dec temp
	STS(adr_DCF_PHASE_TIMER),temp
TIMER_PHASE_w:
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
DIGIT:
; ----------------------------------
	LDS temp,(adr_LED_DIGIT_ZAEHLER)	; ZHLER laden		
	inc temp							; +1	
	; HELLIGKEIT	
	LDS temp1,(adr_LED_HELLIGKEITSWERT)
	cp temp,temp1						; vergleich 		
	brlo DIGIT_w						; ist kleiner		
	clr temp
DIGIT_w:
	STS (adr_LED_DIGIT_ZAEHLER),temp	; ZHLER schreiben	
; ----------------------------------
	; ALLE DIGIT aus	
	cbi PORT_DIG_1,PIN_DIG_1
	cbi PORT_DIG_2,PIN_DIG_2	
	cbi PORT_DIG_3,PIN_DIG_3	
	cbi PORT_DIG_4,PIN_DIG_4
	cbi PORT_DIG_5,PIN_DIG_5	
	cbi PORT_DIG_6,PIN_DIG_6	

	sbi PORT_BALKEN_A,PIN_BALKEN_A	
	sbi PORT_BALKEN_B,PIN_BALKEN_B	
	sbi PORT_BALKEN_C,PIN_BALKEN_C	
	sbi PORT_BALKEN_D,PIN_BALKEN_D	
	sbi PORT_BALKEN_E,PIN_BALKEN_E	
	sbi PORT_BALKEN_F,PIN_BALKEN_F	
	sbi PORT_BALKEN_G,PIN_BALKEN_G	
	sbi PORT_BALKEN_P,PIN_BALKEN_P
	; ZHLER			
	cpi temp,0
	breq DIGIT_0
	cpi temp,1
	breq DIGIT_1
	cpi temp,2
	breq DIGIT_2
	cpi temp,3
	breq DIGIT_3
	cpi temp,4
	breq DIGIT_4
	cpi temp,5
	breq DIGIT_5
	ret
DIGIT_0:
	LDS temp1,(adr_LED_BYTE_0)
	rcall BALKEN
	sbi PORT_DIG_1,PIN_DIG_1	
	ret
DIGIT_1:
	LDS temp1,(adr_LED_BYTE_1)
	rcall BALKEN
	sbi PORT_DIG_2,PIN_DIG_2	
	ret
DIGIT_2:
	LDS temp1,(adr_LED_BYTE_2)
	rcall BALKEN
	sbi PORT_DIG_3,PIN_DIG_3	
	ret
DIGIT_3:
	LDS temp1,(adr_LED_BYTE_3)
	rcall BALKEN
	sbi PORT_DIG_4,PIN_DIG_4	
	ret
DIGIT_4:
	LDS temp1,(adr_LED_BYTE_4)
	rcall BALKEN
	sbi PORT_DIG_5,PIN_DIG_5	
	ret
DIGIT_5:
	LDS temp1,(adr_LED_BYTE_5)
	rcall BALKEN
	sbi PORT_DIG_6,PIN_DIG_6	
	ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
BALKEN:
;-------------------------------------------------------------------------------
BALKEN_A:
	mov temp,temp1
	andi temp,0b00000001
	tst temp
	breq BALKEN_B
	cbi PORT_BALKEN_A,PIN_BALKEN_A	
;-------------------------------------------------------------------------------
BALKEN_B:
	mov temp,temp1
	andi temp,0b00000010
	tst temp
	breq BALKEN_C
	cbi PORT_BALKEN_B,PIN_BALKEN_B	
;-------------------------------------------------------------------------------
BALKEN_C:
	mov temp,temp1
	andi temp,0b00000100
	tst temp
	breq BALKEN_D
	cbi PORT_BALKEN_C,PIN_BALKEN_C	
;-------------------------------------------------------------------------------
BALKEN_D:
	mov temp,temp1
	andi temp,0b00001000
	tst temp
	breq BALKEN_E
	cbi PORT_BALKEN_D,PIN_BALKEN_D	
;-------------------------------------------------------------------------------
BALKEN_E:
	mov temp,temp1
	andi temp,0b00010000
	tst temp
	breq BALKEN_F
	cbi PORT_BALKEN_E,PIN_BALKEN_E	
;-------------------------------------------------------------------------------
BALKEN_F:
	mov temp,temp1
	andi temp,0b00100000
	tst temp
	breq BALKEN_G
	cbi PORT_BALKEN_F,PIN_BALKEN_F	
;-------------------------------------------------------------------------------
BALKEN_G:
	mov temp,temp1
	andi temp,0b01000000
	tst temp
	breq BALKEN_P
	cbi PORT_BALKEN_G,PIN_BALKEN_G	
;-------------------------------------------------------------------------------
BALKEN_P:
	mov temp,temp1
	andi temp,0b10000000
	tst temp
	breq BALKEN_FERTIG
	cbi PORT_BALKEN_P,PIN_BALKEN_P	
;-------------------------------------------------------------------------------
BALKEN_FERTIG:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

